@@ -773,14 +773,14 @@ def test_databag_string_stripping(sentry_init, capture_events, benchmark):
     def inner():
         del events[:]
         try:
-            a = "A" * 1000000  # noqa
+            a = "A" * DEFAULT_MAX_VALUE_LENGTH * 10  # noqa
             1 / 0
         except Exception:
             capture_exception()
 
         (event,) = events
 
-        assert len(json.dumps(event)) < 10000
+        assert len(json.dumps(event)) < DEFAULT_MAX_VALUE_LENGTH * 10
 
 
 def test_databag_breadth_stripping(sentry_init, capture_events, benchmark):
@@ -1073,7 +1073,10 @@ def test_multiple_positional_args(sentry_init):
     "sdk_options, expected_data_length",
     [
         ({}, DEFAULT_MAX_VALUE_LENGTH),
-        ({"max_value_length": 1800}, 1800),
+        (
+            {"max_value_length": DEFAULT_MAX_VALUE_LENGTH + 1000},
+            DEFAULT_MAX_VALUE_LENGTH + 1000,
+        ),
     ],
 )
 def test_max_value_length_option(
@@ -1082,7 +1085,7 @@ def test_max_value_length_option(
     sentry_init(sdk_options)
     events = capture_events()
 
-    capture_message("a" * 2000)
+    capture_message("a" * (DEFAULT_MAX_VALUE_LENGTH + 2000))
 
     assert len(events[0]["message"]) == expected_data_length
 
@@ -1297,7 +1300,6 @@ def test_error_sampler(_, sentry_init, capture_events, test_config):
         assert len(test_config.sampler_function_mock.call_args[0]) == 2
 
 
-@pytest.mark.forked
 @pytest.mark.parametrize(
     "opt,missing_flags",
     [
@@ -1342,6 +1344,8 @@ class TestSpanClientReports:
     Tests for client reports related to spans.
     """
 
+    __test__ = False
+
     @staticmethod
     def span_dropper(spans_to_drop):
         """
